qx.Class.define("pf.base.Bohr",
{
    statics :
    {
        // Fine-structure constant
        alpha  : 7.297352570e-3,
        
        // alpha^2
        alpha2 : 5.325135e-05,
        
        // alpha^3
        alpha3 : 3.885939e-07,
        
        // Speed of light (1/alpha)
        c      : 137.036,
        
        // Bohr magneton (alpha/2), must be in sync with B0!
        mu_B   : 3.64867635e-3,
        
        // Proton mass
        m_p    : 1836.2,
        
        // Electron mass (g)
        m0 : 9.1094e-28,
        // Hartree (eV)
        E0 : 27.2113845,
        // Bohr radius (cm)
        a0 : 5.291772108e-9,
        // Atomic unit time (s)
        t0 : 2.418884327e-17,
        // Atomic unit velocity (cm/s)
        v0 : 2.1876912633e8,
        // Electric field unit (V/cm)
        F0 : 5.14220632e9,
        // Magnetic field unit (T), must be in sync with mu_B!
        B0 : 1.7152553e3,
        // Volume unit (a0^3)
        V0 : 1.481847e-25,
        // Particle density unit (1/V0)
        n0 : 6.748334e+24,
        // Frequency unit (Hz)
        f0 : 4.1341373e16,
        // Pressure unit (dyne/cm^2)
        P0 : 2.9421912e14,
        
        bindingEnergy : function(Z_core, m_core, n)
        {
            return -Z_core*Z_core/(n*n)/2/(1 + 1/m_core);
        },

        transitionEnergy : function(Z_core, m_core, n_u, n_l)
        {
            return this.bindingEnergy(Z_core, m_core, n_u) -
                   this.bindingEnergy(Z_core, m_core, n_l);
        },
        
        fineStructureCorrection : function(Z_core, n, j)
        {
            return -Math.pow(Z_core, 4)*(this.alpha*this.alpha)/
                (n*n*n)*(1.0/(j + 0.5) - 3.0/(4*n))/2;
        },
        
        fineStructureSplitting : function(Z_core, n)
        {
            return this.fineStructureCorrection(Z_core, n, n - 0.5) -
                   this.fineStructureCorrection(Z_core, n, 0.5);
        },
        
        transitionFineStructure : function(Z_core, n_u, n_l)
        {
            return this.fineStructureSplitting(Z_core, n_u) +
                   this.fineStructureSplitting(Z_core, n_l);
        },

        /* Einstein A coefficient of Lyman series of H */
        LymanA : function(n)
        {
            if (n <= 1) {
                return 0;
            } else {
                return Math.pow(this.alpha, 3)*
                   128/9*n*Math.pow((n - 1)/(n + 1), 2*n)/((n*n - 1)*(n*n - 1));
            }
        },

        /* Quasi-classical absorption oscillator strength */
        KramersF: function(n_u, n_l)
        {
            return 32/(3*Math.PI*Math.sqrt(3))*
                n_u*n_u*n_u*n_l/Math.pow(n_u*n_u - n_l*n_l, 3);
        },

        fGauntFactor : function(n_u, n_l)
        {
            var g = 1;
            
            if (n_u == 2 && n_l == 1) {
                g = 0.717;
            } else
            if (n_u == 3 && n_l == 1) {
                g = 0.765;
            } else
            if (n_u == 3 && n_l == 2) {
                g = 0.757;
            } else
            if (n_u - n_l == 1) {
                g = 0.782;
            } else {
                var r2 = (n_l/n_u)*(n_l/n_u);
                var t = Math.pow((1 - r2)*n_l, 2/3);
                g = 1 - 0.1728*(1 + r2)/t;
                     // - 0.0496*(1 + 484/15*r2 + r2*r2)/(t*t);
            }
            
            return g;
        },

        oscillatorStrength : function(n_u, n_l)
        {
            if (n_l == 1) {
                return 256/3*Math.pow((n_u - 1)/(n_u + 1), 2*n_u)/
                       Math.pow(1 - 1/(n_u*n_u), 4)/Math.pow(n_u, 3);
            } else {
                var g = this.fGauntFactor(n_u, n_l);

                return g*this.KramersF(n_u, n_l);
            }
        },
        
        EinsteinA : function(Z_core, m_core, n_u, n_l)
        {
            var f  = this.oscillatorStrength(n_u, n_l);
            var dE = this.transitionEnergy(Z_core, m_core, n_u, n_l);
            
            return 2*this.alpha3*dE*dE*f*(n_l/n_u)*(n_l/n_u);
        },
        
        classicalIonizationField : function(Z_core, n)
        {
            return Math.pow(Z_core, 3)/(16*Math.pow(n, 4));
        }
    }
});
